@@ -1,19 +1,3 @@
-class PrescriptionError(Exception):
-    pass
-
-
-class PrescriptionNotStarted(Exception):
-    pass
-
-
-class PrescriptionExpired(Exception):
-    pass
-
-
-class PrescriptionAlreadyExists(Exception):
-    pass
-
-
 class ActivePrescriptionRefillOverlap(Exception):
     pass
 
@@ -22,10 +6,6 @@ class RefillError(Exception):
     pass
 
 
-class RefillAlreadyExists(Exception):
-    pass
-
-
 class ActiveRefillAlreadyExists(Exception):
     pass
 
@@ -34,6 +14,10 @@ class NextRefillError(Exception):
     pass
 
 
+class NextStudyMedicationError(Exception):
+    pass
+
+
 class InsufficientQuantityError(Exception):
     pass
 
@@ -48,3 +32,35 @@ class PackagingSubjectIdentifierMismatchError(Exception):
 
 class PackagingLotNumberMismatchError(Exception):
     pass
+
+
+class PrescriptionAlreadyExists(Exception):
+    pass
+
+
+class PrescriptionError(Exception):
+    pass
+
+
+class PrescriptionExpired(Exception):
+    pass
+
+
+class RefillAlreadyExists(Exception):
+    pass
+
+
+class PrescriptionNotStarted(Exception):
+    pass
+
+
+class RefillCreatorError(Exception):
+    pass
+
+
+class RefillEndDatetimeError(Exception):
+    pass
+
+
+class StudyMedicationError(Exception):
+    pass
