@@ -1,6 +1,7 @@
 from django.db import models
 from django.db.models import PROTECT
-from edc_model import models as edc_models
+from django_crypto_fields.fields import EncryptedCharField
+from edc_model.models import BaseUuidModel, HistoricalRecords
 
 from .formulation import Formulation
 
@@ -10,21 +11,23 @@ class Manager(models.Manager):
     use_in_migrations = True
 
 
-class MedicationLot(edc_models.BaseUuidModel):
+class MedicationLot(BaseUuidModel):
 
     lot_no = models.CharField(max_length=50, unique=True)
 
+    assignment = EncryptedCharField(null=True)
+
     expiration_date = models.DateField()
 
     formulation = models.ForeignKey(Formulation, on_delete=PROTECT)
 
     objects = Manager()
 
-    history = edc_models.HistoricalRecords()
+    history = HistoricalRecords()
 
     def __str__(self):
-        return self.lot_no
+        return f"{self.formulation} Lot {self.lot_no}"
 
-    class Meta(edc_models.BaseUuidModel.Meta):
+    class Meta(BaseUuidModel.Meta):
         verbose_name = "Medication lot"
         verbose_name_plural = "Medication lots"
