@@ -1,5 +1,5 @@
 from django.db import models
-from edc_crf.crf_model_mixin import CrfModelMixin
+from edc_crf.model_mixins import CrfModelMixin
 from edc_model import models as edc_models
 from edc_sites.models import SiteModelMixin
 from edc_utils import get_utcnow
