@@ -9,17 +9,16 @@ from edc_appointment.creators import AppointmentsCreator, UnscheduledAppointment
 from edc_appointment.models import Appointment
 from edc_appointment.tests.helper import Helper
 from edc_appointment.utils import get_next_appointment
-from edc_constants.constants import NO, YES
+from edc_consent.tests.consent_test_utils import consent_object_factory
+from edc_constants.constants import YES
 from edc_facility import import_holidays
+from edc_protocol import Protocol
 from edc_registration.models import RegisteredSubject
 from edc_utils import get_utcnow
 from edc_visit_schedule import site_visit_schedules
 from edc_visit_tracking.constants import SCHEDULED, UNSCHEDULED
 
-from edc_pharmacy.model_mixins.study_medication_crf_model_mixin import (
-    NextStudyMedicationError,
-    StudyMedicationError,
-)
+from edc_pharmacy.exceptions import NextStudyMedicationError, StudyMedicationError
 from edc_pharmacy.models import (
     DosageGuideline,
     Formulation,
@@ -57,6 +56,7 @@ class TestMedicationCrf(TestCase):
         RegisteredSubject.objects.create(
             subject_identifier=self.subject_identifier,
             registration_datetime=self.registration_datetime,
+            consent_datetime=self.registration_datetime,
         )
         self.helper = self.helper_cls(
             subject_identifier=self.subject_identifier,
@@ -133,9 +133,6 @@ class TestMedicationCrf(TestCase):
             ).appt_datetime,
             dosage_guideline=self.dosage_guideline_100,
             formulation=self.formulation,
-            order_or_update_next=YES,
-            next_dosage_guideline=self.dosage_guideline_200,
-            next_formulation=self.formulation,
         )
         obj.save()
 
@@ -171,9 +168,6 @@ class TestMedicationCrf(TestCase):
             ).appt_datetime,
             dosage_guideline=self.dosage_guideline_100,
             formulation=self.formulation,
-            order_or_update_next=YES,
-            next_dosage_guideline=self.dosage_guideline_200,
-            next_formulation=self.formulation,
         )
         with self.assertRaises(StudyMedicationError):
             obj.save()
@@ -199,9 +193,6 @@ class TestMedicationCrf(TestCase):
             + relativedelta(years=1),
             dosage_guideline=self.dosage_guideline_100,
             formulation=self.formulation,
-            order_or_update_next=YES,
-            next_dosage_guideline=self.dosage_guideline_200,
-            next_formulation=self.formulation,
         )
         with self.assertRaises(StudyMedicationError):
             obj.save()
@@ -227,9 +218,6 @@ class TestMedicationCrf(TestCase):
                     ).appt_datetime,
                     dosage_guideline=self.dosage_guideline_100,
                     formulation=self.formulation,
-                    next_dosage_guideline=None,
-                    next_formulation=None,
-                    order_or_update_next=NO,
                 )
         self.assertEqual(
             StudyMedication.objects.all().count(), Appointment.objects.all().count() - 1
@@ -253,9 +241,6 @@ class TestMedicationCrf(TestCase):
                     ).appt_datetime,
                     dosage_guideline=self.dosage_guideline_100,
                     formulation=self.formulation,
-                    next_dosage_guideline=None,
-                    next_formulation=None,
-                    order_or_update_next=NO,
                 )
 
         # check dates
@@ -292,9 +277,6 @@ class TestMedicationCrf(TestCase):
                     - relativedelta(minutes=1),
                     dosage_guideline=self.dosage_guideline_100,
                     formulation=self.formulation,
-                    next_dosage_guideline=None,
-                    next_formulation=None,
-                    order_or_update_next=NO,
                 )
         obj0 = StudyMedication.objects.all().order_by("refill_start_datetime")[0]
         obj1 = StudyMedication.objects.all().order_by("refill_start_datetime")[1]
@@ -319,9 +301,6 @@ class TestMedicationCrf(TestCase):
                     - relativedelta(minutes=1),
                     dosage_guideline=self.dosage_guideline_100,
                     formulation=self.formulation,
-                    next_dosage_guideline=None,
-                    next_formulation=None,
-                    order_or_update_next=NO,
                 )
 
         for appointment in Appointment.objects.all():
@@ -355,9 +334,6 @@ class TestMedicationCrf(TestCase):
             refill_end_datetime=appointment_after.appt_datetime,
             dosage_guideline=self.dosage_guideline_100,
             formulation=self.formulation,
-            next_dosage_guideline=None,
-            next_formulation=None,
-            order_or_update_next=NO,
         )
         study_medication.save()
 
@@ -405,9 +381,6 @@ class TestMedicationCrf(TestCase):
                     refill_end_datetime=None,
                     dosage_guideline=self.dosage_guideline_100,
                     formulation=self.formulation,
-                    order_or_update_next=YES,
-                    next_dosage_guideline=self.dosage_guideline_100,
-                    next_formulation=self.formulation,
                 )
             else:
                 StudyMedication.objects.create(
@@ -419,54 +392,17 @@ class TestMedicationCrf(TestCase):
                     ).appt_datetime,
                     dosage_guideline=self.dosage_guideline_100,
                     formulation=self.formulation,
-                    order_or_update_next=YES,
-                    next_dosage_guideline=self.dosage_guideline_100,
-                    next_formulation=self.formulation,
                 )
 
-    def test_refill_creates_next_refill(self):
-        appointment = Appointment.objects.all().order_by("timepoint")[0]
-        subject_visit = SubjectVisit.objects.create(
-            appointment=appointment,
-            report_datetime=appointment.appt_datetime,
-            reason=SCHEDULED,
-        )
-        self.assertEqual(RxRefill.objects.all().count(), 0)
-        StudyMedication.objects.create(
-            subject_visit=subject_visit,
-            report_datetime=subject_visit.report_datetime,
-            refill_start_datetime=subject_visit.report_datetime,
-            refill_end_datetime=get_next_appointment(
-                subject_visit.appointment, include_interim=True
-            ).appt_datetime,
-            dosage_guideline=self.dosage_guideline_100,
-            formulation=self.formulation,
-            order_or_update_next=YES,
-            next_dosage_guideline=self.dosage_guideline_200,
-            next_formulation=self.formulation,
-        )
-        self.assertEqual(RxRefill.objects.all().count(), 2)
-
-    def test_refill_creates_next_refill_for_next_dosage(self):
-        appointment = Appointment.objects.all().order_by("timepoint")[0]
-        subject_visit = SubjectVisit.objects.create(
-            appointment=appointment, report_datetime=get_utcnow(), reason=SCHEDULED
-        )
-        StudyMedication.objects.create(
-            subject_visit=subject_visit,
-            report_datetime=subject_visit.report_datetime,
-            refill_start_datetime=subject_visit.report_datetime,
-            refill_end_datetime=get_next_appointment(
-                subject_visit.appointment, include_interim=True
-            ).appt_datetime,
-            dosage_guideline=self.dosage_guideline_100,
-            formulation=self.formulation,
-            order_or_update_next=YES,
-            next_dosage_guideline=self.dosage_guideline_200,
-            next_formulation=self.formulation,
-        )
-
     def test_study_medication_form_baseline(self):
+        self.study_open_datetime = Protocol().study_open_datetime
+        self.study_close_datetime = Protocol().study_close_datetime
+        consent_object_factory(
+            model="edc_pharmacy.subjectconsent",
+            start=self.study_open_datetime,
+            end=self.study_close_datetime,
+        )
+
         appointment = Appointment.objects.all().order_by("timepoint")[0]
         next_appointment = get_next_appointment(appointment, include_interim=True)
         subject_visit = SubjectVisit.objects.create(
@@ -476,15 +412,13 @@ class TestMedicationCrf(TestCase):
         )
         data = dict(
             subject_visit=subject_visit,
+            refill=YES,
             report_datetime=subject_visit.report_datetime,
             refill_start_datetime=subject_visit.report_datetime,
             refill_end_datetime=next_appointment.appt_datetime,
             dosage_guideline=self.dosage_guideline_100,
             formulation=self.formulation,
             refill_to_next_visit=YES,
-            order_or_update_next=YES,
-            next_dosage_guideline=self.dosage_guideline_200,
-            next_formulation=self.formulation,
             roundup_divisible_by=32,
         )
 
@@ -492,43 +426,6 @@ class TestMedicationCrf(TestCase):
         form.is_valid()
         self.assertEqual({}, form._errors)
 
-    def test_study_medication_form_not_order_or_update_next(self):
-        appointment = Appointment.objects.all().order_by("timepoint")[0]
-        subject_visit = SubjectVisit.objects.create(
-            appointment=appointment,
-            report_datetime=appointment.appt_datetime,
-            reason=SCHEDULED,
-        )
-        data = dict(
-            subject_visit=subject_visit,
-            report_datetime=subject_visit.report_datetime,
-            refill_start_datetime=subject_visit.report_datetime,
-            refill_end_datetime=get_next_appointment(
-                subject_visit.appointment, include_interim=True
-            ).appt_datetime,
-            dosage_guideline=self.dosage_guideline_100,
-            formulation=self.formulation,
-            refill_to_next_visit=YES,
-            order_or_update_next=NO,
-            next_dosage_guideline=self.dosage_guideline_200,
-            next_formulation=self.formulation,
-            roundup_divisible_by=32,
-        )
-
-        form = StudyMedicationForm(data=data)
-        form.is_valid()
-        self.assertIn("next_dosage_guideline", form._errors)
-
-        data.update(next_dosage_guideline=None)
-        form = StudyMedicationForm(data=data)
-        form.is_valid()
-        self.assertIn("next_formulation", form._errors)
-
-        data.update(next_formulation=None)
-        form = StudyMedicationForm(data=data)
-        form.is_valid()
-        self.assertEqual({}, form._errors)
-
     def test_inserts_refill(self):
         # 1000
         appointment = Appointment.objects.all().order_by("timepoint")[0]
@@ -547,14 +444,10 @@ class TestMedicationCrf(TestCase):
             ).appt_datetime,
             dosage_guideline=self.dosage_guideline_100,
             formulation=self.formulation,
-            order_or_update_next=YES,
-            next_dosage_guideline=self.dosage_guideline_200,
-            next_formulation=self.formulation,
         )
-        self.assertEqual(RxRefill.objects.all().count(), 2)
+        self.assertEqual(RxRefill.objects.all().count(), 1)
         refills = RxRefill.objects.all().order_by("refill_start_datetime")
         self.assertEqual(refills[0].dosage_guideline, self.dosage_guideline_100)
-        self.assertEqual(refills[1].dosage_guideline, self.dosage_guideline_200)
 
         appointment.appt_status = INCOMPLETE_APPT
         appointment.save()
@@ -576,17 +469,13 @@ class TestMedicationCrf(TestCase):
             ).appt_datetime,
             dosage_guideline=self.dosage_guideline_200,
             formulation=self.formulation,
-            order_or_update_next=YES,
-            next_dosage_guideline=self.dosage_guideline_200,
-            next_formulation=self.formulation,
         )
 
-        self.assertEqual(RxRefill.objects.all().count(), 3)
+        self.assertEqual(RxRefill.objects.all().count(), 2)
 
         refills = RxRefill.objects.all().order_by("refill_start_datetime")
         self.assertEqual(refills[0].dosage_guideline, self.dosage_guideline_100)
         self.assertEqual(refills[1].dosage_guideline, self.dosage_guideline_200)
-        self.assertEqual(refills[2].dosage_guideline, self.dosage_guideline_200)
         appointment.appt_status = INCOMPLETE_APPT
         appointment.save()
 
@@ -605,19 +494,15 @@ class TestMedicationCrf(TestCase):
             refill_end_datetime=None,
             dosage_guideline=self.dosage_guideline_200,
             formulation=self.formulation,
-            order_or_update_next=YES,
-            next_dosage_guideline=self.dosage_guideline_200,
-            next_formulation=self.formulation,
         )
         self.assertRaises(NextStudyMedicationError, StudyMedication.objects.create, **opts)
         appointment.appt_status = INCOMPLETE_APPT
         appointment.save()
 
-        self.assertEqual(RxRefill.objects.all().count(), 3)
+        self.assertEqual(RxRefill.objects.all().count(), 2)
         refills = RxRefill.objects.all().order_by("refill_start_datetime")
         self.assertEqual(refills[0].dosage_guideline, self.dosage_guideline_100)
         self.assertEqual(refills[1].dosage_guideline, self.dosage_guideline_200)
-        self.assertEqual(refills[2].dosage_guideline, self.dosage_guideline_200)
 
         # insert unscheduled appt between 2000 and 3000
 
@@ -646,9 +531,6 @@ class TestMedicationCrf(TestCase):
             ).appt_datetime,
             dosage_guideline=self.dosage_guideline_100,
             formulation=self.formulation,
-            order_or_update_next=YES,
-            next_dosage_guideline=self.dosage_guideline_100,
-            next_formulation=self.formulation,
         )
 
         self.assertEqual(StudyMedication.objects.all().count(), 3)
